@@ -17,13 +17,13 @@ typedef KHash2Map<QPersistentModelIndex, int> Mapping;
 class KDescendantsProxyModelPrivate
 {
     KDescendantsProxyModelPrivate(KDescendantsProxyModel *qq)
-        : q_ptr(qq),
-          m_rowCount(0),
-          m_ignoreNextLayoutAboutToBeChanged(false),
-          m_ignoreNextLayoutChanged(false),
-          m_relayouting(false),
-          m_displayAncestorData(false),
-          m_ancestorSeparator(QStringLiteral(" / "))
+        : q_ptr(qq)
+        , m_rowCount(0)
+        , m_ignoreNextLayoutAboutToBeChanged(false)
+        , m_ignoreNextLayoutChanged(false)
+        , m_relayouting(false)
+        , m_displayAncestorData(false)
+        , m_ancestorSeparator(QStringLiteral(" / "))
     {
     }
 
@@ -128,7 +128,7 @@ void KDescendantsProxyModelPrivate::processPendingParents()
 
         const int rowCount = q->sourceModel()->rowCount(sourceParent);
 
-        // A node can be marked as collapsed or expanded even if it doesn'0t have children
+        // A node can be marked as collapsed or expanded even if it doesn't have children
         if (rowCount == 0) {
             it = m_pendingParents.erase(it);
             continue;
@@ -161,11 +161,8 @@ void KDescendantsProxyModelPrivate::processPendingParents()
             const QModelIndex child = q->sourceModel()->index(sourceRow, column, sourceParent);
             Q_ASSERT(child.isValid());
 
-            if (q->sourceModel()->hasChildren(child)) {
-                Q_ASSERT(q->sourceModel()->rowCount(child) > 0);
-                if (q->isSourceIndexExpanded(child)) {
-                    newPendingParents.append(child);
-                }
+            if (q->sourceModel()->hasChildren(child) && q->isSourceIndexExpanded(child) && q->sourceModel()->rowCount(child) > 0) {
+                newPendingParents.append(child);
             }
         }
     }
@@ -173,7 +170,7 @@ void KDescendantsProxyModelPrivate::processPendingParents()
     if (!m_pendingParents.isEmpty()) {
         processPendingParents();
     }
-//   scheduleProcessPendingParents();
+    //   scheduleProcessPendingParents();
 }
 
 void KDescendantsProxyModelPrivate::updateInternalIndexes(int start, int offset)
@@ -198,11 +195,11 @@ void KDescendantsProxyModelPrivate::updateInternalIndexes(int start, int offset)
             m_mapping.insert(it.value(), it.key());
         }
     }
-
 }
 
 KDescendantsProxyModel::KDescendantsProxyModel(QObject *parent)
-    : QAbstractProxyModel(parent), d_ptr(new KDescendantsProxyModelPrivate(this))
+    : QAbstractProxyModel(parent)
+    , d_ptr(new KDescendantsProxyModelPrivate(this))
 {
 }
 
@@ -242,7 +239,7 @@ bool KDescendantsProxyModel::isSourceIndexExpanded(const QModelIndex &sourceInde
     // Root is always expanded
     if (!sourceIndex.isValid()) {
         return true;
-    } else if (d_ptr->m_expandsByDefault)  {
+    } else if (d_ptr->m_expandsByDefault) {
         return !d_ptr->m_collapsedSourceIndexes.contains(QPersistentModelIndex(sourceIndex));
     } else {
         return d_ptr->m_expandedSourceIndexes.contains(QPersistentModelIndex(sourceIndex));
@@ -274,7 +271,7 @@ void KDescendantsProxyModel::expandSourceIndex(const QModelIndex &sourceIndex)
     }
 
     if (d_ptr->m_expandsByDefault) {
-        d_ptr->m_collapsedSourceIndexes.remove( QPersistentModelIndex(sourceIndex));
+        d_ptr->m_collapsedSourceIndexes.remove(QPersistentModelIndex(sourceIndex));
     } else {
         d_ptr->m_expandedSourceIndexes << QPersistentModelIndex(sourceIndex);
     }
@@ -297,8 +294,8 @@ void KDescendantsProxyModel::collapseSourceIndex(const QModelIndex &sourceIndex)
     const int rowStart = row + 1;
     int rowEnd = row;
 
-    QList <QModelIndex> toVisit = {sourceIndex};
-    QSet <QModelIndex> visited;
+    QList<QModelIndex> toVisit = {sourceIndex};
+    QSet<QModelIndex> visited;
     while (!toVisit.isEmpty()) {
         QModelIndex index = toVisit.takeLast();
         if (!visited.contains(index)) {
@@ -317,25 +314,26 @@ void KDescendantsProxyModel::collapseSourceIndex(const QModelIndex &sourceIndex)
     if (d_ptr->m_expandsByDefault) {
         d_ptr->m_collapsedSourceIndexes << QPersistentModelIndex(sourceIndex);
     } else {
-        d_ptr->m_expandedSourceIndexes.remove( QPersistentModelIndex(sourceIndex));
+        d_ptr->m_expandedSourceIndexes.remove(QPersistentModelIndex(sourceIndex));
     }
 
     {
         Mapping::right_iterator it = d_ptr->m_mapping.rightLowerBound(rowStart);
         const Mapping::right_iterator endIt = d_ptr->m_mapping.rightUpperBound(rowEnd);
 
-        if (endIt != d_ptr->m_mapping.rightEnd())
+        if (endIt != d_ptr->m_mapping.rightEnd()) {
             while (it != endIt) {
                 it = d_ptr->m_mapping.eraseRight(it);
             }
-        else
+        } else {
             while (it != d_ptr->m_mapping.rightUpperBound(rowEnd)) {
                 it = d_ptr->m_mapping.eraseRight(it);
             }
+        }
     }
 
     d_ptr->m_removePair = qMakePair(rowStart, rowEnd);
-    
+
     beginRemoveRows(QModelIndex(), rowStart, rowEnd);
     d_ptr->synchronousMappingRefresh();
     endRemoveRows();
@@ -357,9 +355,10 @@ QModelIndexList KDescendantsProxyModel::match(const QModelIndex &start, int role
     return QAbstractProxyModel::match(start, role, value, hits, flags);
 }
 
-namespace {
-    // we only work on DisplayRole for now
-    static const QVector<int> changedRoles = {Qt::DisplayRole};
+namespace
+{
+// we only work on DisplayRole for now
+static const QVector<int> changedRoles = {Qt::DisplayRole};
 }
 
 void KDescendantsProxyModel::setDisplayAncestorData(bool display)
@@ -370,7 +369,7 @@ void KDescendantsProxyModel::setDisplayAncestorData(bool display)
     if (displayChanged) {
         Q_EMIT displayAncestorDataChanged();
         // send out big hammer. Everything needs to be updated.
-        Q_EMIT dataChanged(index(0,0),index(rowCount()-1,columnCount()-1),  changedRoles);
+        Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, columnCount() - 1), changedRoles);
     }
 }
 
@@ -389,7 +388,7 @@ void KDescendantsProxyModel::setAncestorSeparator(const QString &separator)
         Q_EMIT ancestorSeparatorChanged();
         if (d->m_displayAncestorData) {
             // send out big hammer. Everything needs to be updated.
-            Q_EMIT dataChanged(index(0,0),index(rowCount()-1,columnCount()-1),  changedRoles);
+            Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, columnCount() - 1), changedRoles);
         }
     }
 }
@@ -406,48 +405,67 @@ void KDescendantsProxyModel::setSourceModel(QAbstractItemModel *_sourceModel)
 
     beginResetModel();
 
-    static const char *const modelSignals[] = {
-        SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
-        SIGNAL(rowsInserted(QModelIndex,int,int)),
-        SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
-        SIGNAL(rowsRemoved(QModelIndex,int,int)),
-        SIGNAL(rowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-        SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
-        SIGNAL(modelAboutToBeReset()),
-        SIGNAL(modelReset()),
-        SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-        SIGNAL(layoutAboutToBeChanged()),
-        SIGNAL(layoutChanged()),
-        SIGNAL(destroyed())
-    };
-    static const char *const proxySlots[] = {
-        SLOT(sourceRowsAboutToBeInserted(QModelIndex,int,int)),
-        SLOT(sourceRowsInserted(QModelIndex,int,int)),
-        SLOT(sourceRowsAboutToBeRemoved(QModelIndex,int,int)),
-        SLOT(sourceRowsRemoved(QModelIndex,int,int)),
-        SLOT(sourceRowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)),
-        SLOT(sourceRowsMoved(QModelIndex,int,int,QModelIndex,int)),
-        SLOT(sourceModelAboutToBeReset()),
-        SLOT(sourceModelReset()),
-        SLOT(sourceDataChanged(QModelIndex,QModelIndex)),
-        SLOT(sourceLayoutAboutToBeChanged()),
-        SLOT(sourceLayoutChanged()),
-        SLOT(sourceModelDestroyed())
-    };
-
     if (sourceModel()) {
-        for (int i = 0; i < int(sizeof modelSignals / sizeof * modelSignals); ++i) {
-            disconnect(sourceModel(), modelSignals[i], this, proxySlots[i]);
-        }
+        disconnect(sourceModel(), nullptr, this, nullptr);
     }
 
     QAbstractProxyModel::setSourceModel(_sourceModel);
     d_ptr->m_expandedSourceIndexes.clear();
 
     if (_sourceModel) {
-        for (int i = 0; i < int(sizeof modelSignals / sizeof * modelSignals); ++i) {
-            connect(_sourceModel, modelSignals[i], this, proxySlots[i]);
-        }
+        connect(_sourceModel, &QAbstractItemModel::rowsAboutToBeInserted, this, [d](const QModelIndex &parent, int start, int end) {
+            d->sourceRowsAboutToBeInserted(parent, start, end);
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::rowsInserted, this, [d](const QModelIndex &parent, int start, int end) {
+            d->sourceRowsInserted(parent, start, end);
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::rowsAboutToBeRemoved, this, [d](const QModelIndex &parent, int start, int end) {
+            d->sourceRowsAboutToBeRemoved(parent, start, end);
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::rowsRemoved, this, [d](const QModelIndex &parent, int start, int end) {
+            d->sourceRowsRemoved(parent, start, end);
+        });
+
+        connect(_sourceModel,
+                &QAbstractItemModel::rowsAboutToBeMoved,
+                this,
+                [d](const QModelIndex &srcParent, int srcStart, int srcEnd, const QModelIndex &destParent, int destStart) {
+                    d->sourceRowsAboutToBeMoved(srcParent, srcStart, srcEnd, destParent, destStart);
+                });
+
+        connect(_sourceModel,
+                &QAbstractItemModel::rowsMoved,
+                this,
+                [d](const QModelIndex &srcParent, int srcStart, int srcEnd, const QModelIndex &destParent, int destStart) {
+                    d->sourceRowsMoved(srcParent, srcStart, srcEnd, destParent, destStart);
+                });
+
+        connect(_sourceModel, &QAbstractItemModel::modelAboutToBeReset, this, [d]() {
+            d->sourceModelAboutToBeReset();
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::modelReset, this, [d]() {
+            d->sourceModelReset();
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::dataChanged, this, [d](const QModelIndex &topLeft, const QModelIndex &bottomRight) {
+            d->sourceDataChanged(topLeft, bottomRight);
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::layoutAboutToBeChanged, this, [d]() {
+            d->sourceLayoutAboutToBeChanged();
+        });
+
+        connect(_sourceModel, &QAbstractItemModel::layoutChanged, this, [d]() {
+            d->sourceLayoutChanged();
+        });
+
+        connect(_sourceModel, &QObject::destroyed, this, [d]() {
+            d->sourceModelDestroyed();
+        });
     }
 
     resetInternalData();
@@ -479,7 +497,6 @@ int KDescendantsProxyModel::rowCount(const QModelIndex &parent) const
     }
 
     if (d->m_mapping.isEmpty() && sourceModel()->hasChildren()) {
-        Q_ASSERT(sourceModel()->rowCount() > 0);
         const_cast<KDescendantsProxyModelPrivate *>(d)->synchronousMappingRefresh();
     }
     return d->m_rowCount;
@@ -597,8 +614,8 @@ QModelIndex KDescendantsProxyModel::mapFromSource(const QModelIndex &sourceIndex
                 index = ancestor;
             }
             if (found_block && !index.isValid())
-                // Looked through the ascendants of it.key() without finding sourceParent.
-                // That means we've already got the result we need.
+            // Looked through the ascendants of it.key() without finding sourceParent.
+            // That means we've already got the result we need.
             {
                 break;
             }
@@ -618,7 +635,6 @@ QModelIndex KDescendantsProxyModel::mapFromSource(const QModelIndex &sourceIndex
         Q_ASSERT(!"Didn't find valid proxy mapping.");
         return QModelIndex();
     }
-
 }
 
 int KDescendantsProxyModel::columnCount(const QModelIndex &parent) const
@@ -745,8 +761,7 @@ void KDescendantsProxyModelPrivate::sourceRowsAboutToBeInserted(const QModelInde
         Q_ASSERT(rowCount == start);
         static const int column = 0;
         QModelIndex idx = q->sourceModel()->index(rowCount - 1, column, parent);
-        while (q->isSourceIndexExpanded(idx) && q->sourceModel()->hasChildren(idx)) {
-            Q_ASSERT(q->sourceModel()->rowCount(idx) > 0);
+        while (q->isSourceIndexExpanded(idx) && q->sourceModel()->hasChildren(idx) && q->sourceModel()->rowCount(idx) > 0) {
             idx = q->sourceModel()->index(q->sourceModel()->rowCount(idx) - 1, column, idx);
         }
         // The last item in the list is getting a sibling below it.
@@ -763,7 +778,9 @@ void KDescendantsProxyModelPrivate::sourceRowsInserted(const QModelIndex &parent
     Q_Q(KDescendantsProxyModel);
     if (parent.isValid() && (!q->isSourceIndexExpanded(parent) || !q->isSourceIndexVisible(parent))) {
         const QModelIndex index = q->mapFromSource(parent);
-        Q_EMIT q->dataChanged(index, index, {KDescendantsProxyModel::ExpandableRole, KDescendantsProxyModel::ExpandedRole, KDescendantsProxyModel::HasSiblingsRole});
+        Q_EMIT q->dataChanged(index,
+                              index,
+                              {KDescendantsProxyModel::ExpandableRole, KDescendantsProxyModel::ExpandedRole, KDescendantsProxyModel::HasSiblingsRole});
         if (start > 0) {
             notifyhasSiblings(q->sourceModel()->index(start - 1, 0, parent));
         }
@@ -779,7 +796,9 @@ void KDescendantsProxyModelPrivate::sourceRowsInserted(const QModelIndex &parent
     if (rowCount == difference) {
         const QModelIndex index = q->mapFromSource(parent);
         if (parent.isValid()) {
-            Q_EMIT q->dataChanged(index, index, {KDescendantsProxyModel::ExpandableRole, KDescendantsProxyModel::ExpandedRole, KDescendantsProxyModel::HasSiblingsRole});
+            Q_EMIT q->dataChanged(index,
+                                  index,
+                                  {KDescendantsProxyModel::ExpandableRole, KDescendantsProxyModel::ExpandedRole, KDescendantsProxyModel::HasSiblingsRole});
         }
         // @p parent was not a parent before.
         m_pendingParents.append(parent);
@@ -849,7 +868,7 @@ void KDescendantsProxyModelPrivate::sourceRowsInserted(const QModelIndex &parent
 
             while (q->isSourceIndexExpanded(indexAbove) && q->sourceModel()->hasChildren(indexAbove)) {
                 Q_ASSERT(q->sourceModel()->rowCount(indexAbove) > 0);
-                indexAbove = q->sourceModel()->index(q->sourceModel()->rowCount(indexAbove) - 1,  column, indexAbove);
+                indexAbove = q->sourceModel()->index(q->sourceModel()->rowCount(indexAbove) - 1, column, indexAbove);
             }
             Q_ASSERT(!q->isSourceIndexExpanded(indexAbove) || q->sourceModel()->rowCount(indexAbove) == 0);
         }
@@ -870,8 +889,7 @@ void KDescendantsProxyModelPrivate::sourceRowsInserted(const QModelIndex &parent
         const QModelIndex idx = q->sourceModel()->index(row, column, parent);
         Q_ASSERT(idx.isValid());
 
-        if (q->isSourceIndexExpanded(idx) && q->sourceModel()->hasChildren(idx)) {
-            Q_ASSERT(q->sourceModel()->rowCount(idx) > 0);
+        if (q->isSourceIndexExpanded(idx) && q->sourceModel()->hasChildren(idx) && q->sourceModel()->rowCount(idx) > 0) {
             m_pendingParents.append(idx);
         }
     }
@@ -882,13 +900,14 @@ void KDescendantsProxyModelPrivate::sourceRowsInserted(const QModelIndex &parent
     scheduleProcessPendingParents();
     if (parent.isValid()) {
         const QModelIndex index = q->mapFromSource(parent);
-        Q_EMIT q->dataChanged(index, index, {KDescendantsProxyModel::ExpandableRole, KDescendantsProxyModel::ExpandedRole, KDescendantsProxyModel::HasSiblingsRole});
+        Q_EMIT q->dataChanged(index,
+                              index,
+                              {KDescendantsProxyModel::ExpandableRole, KDescendantsProxyModel::ExpandedRole, KDescendantsProxyModel::HasSiblingsRole});
     }
 
     if (start > 0) {
         notifyhasSiblings(q->sourceModel()->index(start - 1, 0, parent));
     }
-
 }
 
 void KDescendantsProxyModelPrivate::sourceRowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
@@ -903,13 +922,12 @@ void KDescendantsProxyModelPrivate::sourceRowsAboutToBeRemoved(const QModelIndex
 
     static const int column = 0;
     QModelIndex idx = q->sourceModel()->index(end, column, parent);
-    while (q->sourceModel()->hasChildren(idx)) {
-        Q_ASSERT(q->sourceModel()->rowCount(idx) > 0);
+    while (q->sourceModel()->hasChildren(idx) && q->sourceModel()->rowCount(idx) > 0) {
         idx = q->sourceModel()->index(q->sourceModel()->rowCount(idx) - 1, column, idx);
     }
     const int proxyEnd = q->mapFromSource(idx).row();
 
-    for(int i = start; i <= end; ++i) {
+    for (int i = start; i <= end; ++i) {
         QModelIndex idx = q->sourceModel()->index(i, column, parent);
         m_expandedSourceIndexes.remove(QPersistentModelIndex(idx));
     }
@@ -958,14 +976,15 @@ void KDescendantsProxyModelPrivate::sourceRowsRemoved(const QModelIndex &parent,
         Mapping::right_iterator it = m_mapping.rightLowerBound(proxyStart);
         const Mapping::right_iterator endIt = m_mapping.rightUpperBound(proxyEnd);
 
-        if (endIt != m_mapping.rightEnd())
+        if (endIt != m_mapping.rightEnd()) {
             while (it != endIt) {
                 it = m_mapping.eraseRight(it);
             }
-        else
+        } else {
             while (it != m_mapping.rightUpperBound(proxyEnd)) {
                 it = m_mapping.eraseRight(it);
             }
+        }
     }
 
     m_removePair = qMakePair(-1, -1);
@@ -1119,20 +1138,24 @@ void KDescendantsProxyModelPrivate::sourceRowsRemoved(const QModelIndex &parent,
     }
 }
 
-void KDescendantsProxyModelPrivate::sourceRowsAboutToBeMoved(const QModelIndex &srcParent, int srcStart, int srcEnd, const QModelIndex &destParent, int destStart)
+void KDescendantsProxyModelPrivate::sourceRowsAboutToBeMoved(const QModelIndex &srcParent,
+                                                             int srcStart,
+                                                             int srcEnd,
+                                                             const QModelIndex &destParent,
+                                                             int destStart)
 {
     Q_Q(KDescendantsProxyModel);
 
     Q_UNUSED(destStart)
 
-    if (q->isSourceIndexExpanded(srcParent) && q->isSourceIndexVisible(srcParent) &&
-        (!q->isSourceIndexExpanded(destParent) || !q->isSourceIndexVisible(destParent))) {
+    if (q->isSourceIndexExpanded(srcParent) && q->isSourceIndexVisible(srcParent)
+        && (!q->isSourceIndexExpanded(destParent) || !q->isSourceIndexVisible(destParent))) {
         const QModelIndex proxySrcParent = q->mapFromSource(srcParent);
         const int proxyParentRow = proxySrcParent.isValid() ? proxySrcParent.row() : 0;
         q->beginRemoveRows(QModelIndex(), proxyParentRow + srcStart, proxyParentRow + srcEnd);
 
-    } else if ((!q->isSourceIndexExpanded(srcParent) || !q->isSourceIndexVisible(srcParent)) &&
-        q->isSourceIndexExpanded(destParent) && q->isSourceIndexVisible(destParent)) {
+    } else if ((!q->isSourceIndexExpanded(srcParent) || !q->isSourceIndexVisible(srcParent)) && q->isSourceIndexExpanded(destParent)
+               && q->isSourceIndexVisible(destParent)) {
         const QModelIndex proxyDestParent = q->mapFromSource(srcParent);
         const int proxyParentRow = proxyDestParent.isValid() ? proxyDestParent.row() : 0;
 
@@ -1152,8 +1175,8 @@ void KDescendantsProxyModelPrivate::sourceRowsMoved(const QModelIndex &srcParent
     Q_UNUSED(destParent)
     Q_UNUSED(destStart)
 
-    if (q->isSourceIndexExpanded(srcParent) && q->isSourceIndexVisible(srcParent) &&
-        (!q->isSourceIndexExpanded(destParent) || !q->isSourceIndexVisible(destParent))) {
+    if (q->isSourceIndexExpanded(srcParent) && q->isSourceIndexVisible(srcParent)
+        && (!q->isSourceIndexExpanded(destParent) || !q->isSourceIndexVisible(destParent))) {
         q->endRemoveRows();
     } else if (!q->isSourceIndexExpanded(srcParent) && q->isSourceIndexExpanded(destParent)) {
         q->endInsertRows();
@@ -1191,8 +1214,7 @@ void KDescendantsProxyModelPrivate::sourceModelReset()
 {
     Q_Q(KDescendantsProxyModel);
     resetInternalData();
-    if (q->sourceModel()->hasChildren()) {
-        Q_ASSERT(q->sourceModel()->rowCount() > 0);
+    if (q->sourceModel()->hasChildren() && q->sourceModel()->rowCount() > 0) {
         m_pendingParents.append(QModelIndex());
         scheduleProcessPendingParents();
     }
@@ -1216,7 +1238,7 @@ void KDescendantsProxyModelPrivate::sourceLayoutAboutToBeChanged()
 
     QPersistentModelIndex srcPersistentIndex;
     const auto lst = q->persistentIndexList();
-    for (const QPersistentModelIndex &proxyPersistentIndex : lst) {
+    for (const QModelIndex &proxyPersistentIndex : lst) {
         m_proxyIndexes << proxyPersistentIndex;
         Q_ASSERT(proxyPersistentIndex.isValid());
         srcPersistentIndex = q->mapToSource(proxyPersistentIndex);
